
#include <stdio.h>
#include <stdlib.h>
#include "cbp.h"
#include "cell.h"
#include "sif.h"

extern void	Error();

#define STR_LEN	256
#define SPRINTF	(void)sprintf

int	main(argc,argv)
int	argc;
char	**argv;
{
	FILE	*file;
	char	str[STR_LEN],r0dr[STR_LEN],*ptr,
		*sdre="sinogram data read error.";
	int	N,M,BPS,m,n,v,h;
	double	dr,r0,t0,f0,df,p,f1,f2,f3;
	Float	**P,**f;
	Cell	c,**cell;

	if (argc!=7 && argc!=9)
	    Error("usage : sg2tg sinogram dr r0/dr t0 {f0 df} BPS tomogram");

	if (argv[1][0]=='-')
	    file=stdin;
	else
	    if ((file=fopen(argv[1],"rb"))==NULL)
		Error("sinogram file not found.");

	if (fgets(str,STR_LEN,file)==NULL ||
	    sscanf(str,"%d %d",&N,&M)!=2 ||
	    N<=0 || M<=0) Error("bad number of sinogram data.");

	if ((dr=atof(argv[2]))<=0.0) Error("bad detector interval.");

	if ((r0=atof(argv[3]))>=0.0 || r0<=(double)(1-N))
	    Error("bad detector offset (unit in detector interval).");

	t0=atof(argv[4])*M_PI/180.0;

	if (argc==9) {
	    f0=atof(argv[5]);
	    if ((df=atof(argv[6]))<=0.0) Error("bad CT-value step.");
	}
	if ((BPS=atoi(argv[argc-2]))<=0 || BPS>8*sizeof(Cell))
	    Error("bad bit per sample (BPS).");

	if ((P=InitCBP(N,M))==NULL) Error("memory allocation error.");

	if (sizeof(double)!=sizeof(Float))
	    for (m=0; m<M; m++)
	    for (n=0; n<N; n++) {
		if (fread(&p,sizeof(p),(size_t)1,file)!=1) Error(sdre);

		P[m][n]=p;
	    }
	else
	    if (fread(*P,sizeof(**P),(size_t)(M*N),file)!=M*N) Error(sdre);

	if (file!=stdin) (void)fclose(file);

	f=CBP(dr,r0,t0);

	f1=f2=(**f);
	for (v=0; v<N; v++)
	for (h=0; h<N; h++)
	    if (f[v][h]<f1) f1=f[v][h]; else if (f[v][h]>f2) f2=f[v][h];

	(void)printf("%lf\t%lf\n",f1,f2);

	SPRINTF(r0dr,"%21.18lf",r0*dr); for (ptr=r0dr; *ptr!='\0'; ptr++) ;
					while (*(--ptr)=='0') *ptr='\0';
					if (*ptr=='.') *ptr='\0';
	c=(Cell)((1<<BPS)-1);

	if (argc==9) {
	    SPRINTF(str,"%s\t%s\n%d\t%s\n%lf\t%lf\n%s\t%s",
			argv[2],r0dr,M,argv[4],f1,f2,argv[5],argv[6]);

	    f3=f0+df*(double)c;
	}
	else {
	    SPRINTF(str,"%s\t%s\n%d\t%s\n%lf\t%lf",
			argv[2],r0dr,M,argv[4],f1,f2);

	    f0=f1; df=(f2-f1)/(double)c; f3=f2;
	}
	f0-=(df/2.0);

	cell=(Cell **)f;
	for (v=0; v<N; v++)
	for (h=0; h<N; h++)
	    cell[v][h]=(f[v][h]<f0)?0:(f[v][h]>f3)?c:(Cell)((f[v][h]-f0)/df);

	StoreImageFile(argv[argc-1],N,N,BPS,cell,str);

	TermCBP(); return 0;
}
